/*********************************************
This program was produced by the
CodeWizardAVR V1.24.0 Standard
Automatic Program Generator
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : Thermomtre I2C
Version : 1
Date    : 29/12/2008
Author  : Philippe Mariano                         
Company : Lyce Pierre Emile Martin                                
Comments: 
Lecture de la temprature dlivr par un DS1621
Affichage sur un LCD et sur des LED, rglage de la temprature
minimum affiche


Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*********************************************/

// ***************************************************************************************
// -------------------------------------Bibliothques ------------------------------------ 
#include <mega32.h>
//#include <mega8535.h>
#include <stdio.h>  	// contient la fonction sprintf()
#include <math.h> 	// contient la fonction abs()
#include <delay.h>  	// contient la fonction delay_ms()  
// ---------------------------------------------------------------------------------------
// I2C Bus functions
#asm
   .equ __i2c_port=0x12
   .equ __sda_bit=3  
   .equ __scl_bit=7  
#endasm
#include <i2c.h>

// DS1621 Thermometer/Thermostat functions
#include <ds1621.h>

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15
#endasm
#include <lcd.h>
// ***************************************************************************************

// ***************************************************************************************
// ---------------------- Octets de commande des composants I2C -------------------------- 

// Dtermination de l'octet de commande des LED: 0 1 0 0 A2 A1 A0 0
#define EcritureLED	0b01000010   					// <---  complter

// Dtermination des octets de commande du SWITCH: 0 1 0 0 A2 A1 A0 R/W			
#define LectureSWITCH	0b01000101					// <---  complter
#define InitSWITCH	0b01000100	

// Dtermination de la partie configurable de l'adresse du capteur 
// temprature (DS1621)
#define A_DS1621		0                   			// <---  complter
// ***************************************************************************************


// ***************************************************************************************
// 				Dbut Programme principal
// ---------------------------------------------------------------------------------------
void main(void)
{
// ***************************************************************************************
// Dclaration des variables locales
// ---------------------------------------------------------------------------------------
// type 		nom 		Commentaires  
// ---------------------------------------------------------------------------------------
char display_buffer[17];     	// Tampon pour la premire ligne de l'afficheur 
unsigned char Tmin=0;		// Temprature mini affiche sur les LED    
int temperature, Ecart;		// Image de la temprature mesure par le DS1621 et
				// cart entre la temprature mesure et la temprature
				// mini affiche sur les LED
unsigned char DipSwitch=0, LED=0xFF;	// Etat du DIP switch et mot de commande des LED
// ***************************************************************************************

// ***************************************************************************************
// 				INITIALISATION DES PERIPHERIQUES
// ***************************************************************************************
// I2C Bus initialization
// ---------------------------------------------------------------------------------------
i2c_init();

// DS1621 Thermometer/Thermostat initialization
// ---------------------------------------------------------------------------------------
// tlow: 50C
// thigh: 55C
// Tout polarity: 0
ds1621_init(0,50,55,0);

// LCD module initialization
// ---------------------------------------------------------------------------------------
lcd_init(16);

// Initialisation du port E/S PCF8574 reli au DIP switch
// ---------------------------------------------------------------------------------------
i2c_start();    		// Adresse des switchs = 2
i2c_write(InitSWITCH); 		// Ecriture de 0
i2c_write(0x00); 
i2c_stop();           

// Initialisation du port E/S PCF8574 reli aux LED
// ---------------------------------------------------------------------------------------
i2c_start(); 			// Adresse des LED = 1 
i2c_write(EcritureLED);  	// Ecriture de FF pour teindre les LED
i2c_write(0xFF); 
i2c_stop();
// ***************************************************************************************


while (1)
      {
// ---------------------------------- Lecture de la temprature ------------------------      
temperature = ds1621_temperature_10(A_DS1621);       

// ---------------------------------- Lecture du DIP switch ----------------------------      
	i2c_start();
	i2c_write(LectureSWITCH);       			
	DipSwitch = i2c_read(0);	
	i2c_stop();  
 
// --------------------------------------Traitement------------------------------------- 
         Tmin = DipSwitch & 0b00011111;      			// <---  complter
         
         Ecart = (temperature/10) - Tmin;          		// <---  complter
         
	 if ((( Ecart > 0) || (Ecart == 0)) && ((Ecart < 7) || (Ecart == 7)))
 	   {
 	     switch (Ecart)     
		{     						// <---  complter
 		     case 0: LED = 0b11111110; break;
 		     case 1: LED = 0b11111100; break;
 		     case 2: LED = 0b11111000; break;
 		     case 3: LED = 0b11110000; break;
 		     case 4: LED = 0b11100000; break;
 		     case 5: LED = 0b11000000; break;
 		     case 6: LED = 0b10000000; break;
 		     case 7: LED = 0b00000000; break; 
 		}
 	   }
 	     else
 		LED = 0b11111111; 
         	
// ----------------------------- Affichage de la temprature-----------------------------
	i2c_start(); 	// Affichage sur les LED
	i2c_write(EcritureLED); 
	i2c_write(LED); 
	i2c_stop(); 
         		// Affichage sur le LCD
	sprintf(display_buffer,"T=%-i.%-u%-cC",temperature/10,abs(temperature%10),0xdf); 
      	lcd_clear();
      	lcd_puts(display_buffer);   
      	delay_ms(1000);    
      };
}
// ---------------------------------------------------------------------------------------
// 				Fin Programme principal
// ***************************************************************************************